@@ -13,9 +13,38 @@ import urllib.parse
 
 mode ='local'
 
+if(mode=='vpn'):
+    def nsefetch(payload):
+        if (("%26" in payload) or ("%20" in payload)):
+            encoded_url = payload
+        else:
+            encoded_url = urllib.parse.quote(payload, safe=':/?&=')
+        payload_var = 'curl -b cookies.txt "' + encoded_url + '"' + curl_headers + ''
+        try:
+            output = os.popen(payload_var).read()
+            output=json.loads(output)
+        except ValueError:  # includes simplejson.decoder.JSONDecodeError:
+            payload2 = "https://www.nseindia.com"
+            output2 = os.popen('curl -c cookies.txt "'+payload2+'"'+curl_headers+'').read()
+    
+            output = os.popen(payload_var).read()
+            output=json.loads(output)
+        return output
 if(mode=='local'):
     def nsefetch(payload):
-        output = requests.get(payload,headers=headers).json()
+        try: #Try the normal way.
+            output = requests.get(payload,headers=headers).json()
+        except: 
+            try: # Follow Praty's hack as mentioned here at https://forum.unofficed.com/t/solution-equity-history-function-doesnt-work/1197
+                output = requests.session().get(payload,headers=headers).json()
+
+            except: #Mimick FrontPage Visit
+                s=requests.session()
+
+                payload2 = "https://www.nseindia.com"
+                output2 = requests.get(payload2,headers=headers).json()
+
+                output = requests.get(payload,headers=headers).json()
         return output
 
 
@@ -579,12 +608,11 @@ def derivative_history_virgin(symbol,start_date,end_date,instrumentType,expiry_d
     instrumentType = instrumentType.lower()
 
     if(instrumentType=="options"):
+        if("NIFTY" in symbol): instrumentType="FUTSTK"
         instrumentType="OPTSTK"
-        if("NIFTY" in symbol): instrumentType="OPTIDX"
-        
     if(instrumentType=="futures"):
-        instrumentType="FUTSTK"
-        if("NIFTY" in symbol): instrumentType="FUTIDX"
+        if("NIFTY" in symbol): instrumentType="OPTIDX"
+        instrumentType="FUTIDX"
 
     if(((instrumentType=="OPTIDX")or (instrumentType=="OPTSTK")) and (expiry_date!="")):
         strikePrice = "%.2f" % strikePrice
@@ -794,3 +822,10 @@ def nse_eq_symbols():
     #https://forum.unofficed.com/t/feature-request-stocklist-api/1073/11
     eq_list_pd = pd.read_csv('https://archives.nseindia.com/content/equities/EQUITY_L.csv')
     return eq_list_pd['SYMBOL'].tolist()
+
+def nse_price_band_hitters(bandtype="both",view="AllSec"):
+  payload = nsefetch("https://www.nseindia.com/api/live-analysis-price-band-hitter")
+  
+  #bandtype can be upper, lower, both
+  #view can be AllSec,SecGtr20,SecLwr20
+  return pd.DataFrame(payload[bandtype][view]["data"])
\ No newline at end of file
